@@ -18,6 +18,11 @@ using std::string;
 
 #include <libnormaliz/libnormaliz.h>
 
+#ifdef ENFNORMALIZ
+using eantic::renf_elem_class;
+using eantic::renf_class;
+#endif 
+
 using libnormaliz::Cone;
 // using libnormaliz::ConeProperty;
 using libnormaliz::ConeProperties;
@@ -172,11 +177,11 @@ static PyObject* CallPythonFuncOnOneArg(PyObject* function, PyObject* single_arg
 #ifndef NMZ_RELEASE
 static_assert(
     false,
-    "Your Normaliz version (unknown) is to old! Update to 3.8.10 or newer.");
+    "Your Normaliz version (unknown) is to old! Update to 3.9.0 or newer.");
 #endif
-#if NMZ_RELEASE < 30810
+#if NMZ_RELEASE < 30900
 static_assert(false,
-              "Your Normaliz version is to old! Update to 3.8.10 or newer.");
+              "Your Normaliz version is to old! Update to 3.9.0 or newer.");
 #endif
 
 /***************************************************************************
@@ -471,8 +476,8 @@ static bool prepare_nf_input(vector< vector< NumberFieldElem > >& out,
                 current_elem = NumberFieldElem(*nf, current_vector);
             }
             if (string_check(current_element)) {
-                current_elem = NumberFieldElem(*nf);
-                current_elem = PyUnicodeToString(current_element);
+                current_elem = NumberFieldElem(*nf,PyUnicodeToString(current_element));
+                // current_elem = PyUnicodeToString(current_element);
             }
             if (PyFloat_Check(current_element)){
                 throw PyNormalizInputException("Nonintegral numbers must be given as strings"); 
@@ -709,27 +714,60 @@ template < typename Integer >
 static PyObject*
 NmzAutomorphismsToPython(const AutomorphismGroup< Integer >& grp)
 {
+    int list_size = 6;
+    if(grp.IsInput() || grp.IsAmbient())
+        list_size =7;    
 
-    PyObject* list = PyList_New(4);
+    PyObject* list = PyList_New(list_size);
 
     PyList_SetItem(list, 0, NmzToPyNumber(grp.getOrder()));
+    PyList_SetItem(list, 1, BoolToPyBool(grp.IsIntegralityChecked()));
+    PyList_SetItem(list, 2, BoolToPyBool(grp.IsIntegral()));
+    
+    if(grp.IsInput() || grp.IsAmbient()){
+        PyList_SetItem(list, 6, NmzMatrixToPyList(grp.getGens().get_elements()));
+        PyObject* current = PyList_New(2);        
+        PyList_SetItem(current, 0, NmzMatrixToPyList(grp.getGensPerms()));
+        PyList_SetItem(current, 1, NmzMatrixToPyList(grp.getGensOrbits()));
+        PyList_SetItem(list, 3, current);
+        
+        current = PyList_New(2);
+        vector<vector<long> > Empty;
+        PyList_SetItem(current, 0, NmzMatrixToPyList(Empty));
+        PyList_SetItem(current, 1, NmzMatrixToPyList(Empty));
+        PyList_SetItem(list, 4, current);
+        
+        if(grp.IsAmbient()){
+            current = PyList_New(2);
+            PyList_SetItem(current, 0, NmzMatrixToPyList(grp.getLinFormsPerms()));
+            PyList_SetItem(current, 1, NmzMatrixToPyList(grp.getLinFormsOrbits()));
+            PyList_SetItem(list, 5, current);            
+        }
+        else{
+            vector<vector<long> > Empty;
+            PyList_SetItem(current, 0, NmzMatrixToPyList(Empty));
+            PyList_SetItem(current, 1, NmzMatrixToPyList(Empty));
+            PyList_SetItem(list, 5, current);            
+        }        
+    }    
+    else{
+        PyObject* current = PyList_New(2);
+        PyList_SetItem(current, 0, NmzMatrixToPyList(grp.getExtremeRaysPerms()));
+        PyList_SetItem(current, 1, NmzMatrixToPyList(grp.getExtremeRaysOrbits()));
+        PyList_SetItem(list, 3, current);
 
-    PyObject* current = PyList_New(2);
-    PyList_SetItem(current, 0, NmzMatrixToPyList(grp.getExtremeRaysPerms()));
-    PyList_SetItem(current, 1, NmzMatrixToPyList(grp.getExtremeRaysOrbits()));
-    PyList_SetItem(list, 1, current);
+        current = PyList_New(2);
+        PyList_SetItem(current, 0, NmzMatrixToPyList(grp.getVerticesPerms()));
+        PyList_SetItem(current, 1, NmzMatrixToPyList(grp.getVerticesOrbits()));
+        PyList_SetItem(list, 4, current);
 
-    current = PyList_New(2);
-    PyList_SetItem(current, 0, NmzMatrixToPyList(grp.getVerticesPerms()));
-    PyList_SetItem(current, 1, NmzMatrixToPyList(grp.getVerticesOrbits()));
-    PyList_SetItem(list, 2, current);
-
-    current = PyList_New(2);
-    PyList_SetItem(current, 0,
-                   NmzMatrixToPyList(grp.getSupportHyperplanesPerms()));
-    PyList_SetItem(current, 1,
-                   NmzMatrixToPyList(grp.getSupportHyperplanesOrbits()));
-    PyList_SetItem(list, 3, current);
+        current = PyList_New(2);
+        PyList_SetItem(current, 0,
+                    NmzMatrixToPyList(grp.getSupportHyperplanesPerms()));
+        PyList_SetItem(current, 1,
+                    NmzMatrixToPyList(grp.getSupportHyperplanesOrbits()));
+        PyList_SetItem(list, 5, current);    
+    }
 
     return list;
 }
@@ -742,7 +780,7 @@ NmzAutomorphismsToPython(const AutomorphismGroup< Integer >& grp)
 
 #ifdef ENFNORMALIZ
 struct NumberFieldCone {
-    renf_class*              nf;
+    const renf_class*              nf;
     Cone< renf_elem_class >* cone;
 };
 #endif
@@ -791,7 +829,7 @@ static Cone< renf_elem_class >* get_cone_renf(PyObject* cone)
     return cone_ptr->cone;
 }
 
-static renf_class* get_cone_renf_renf(PyObject* cone)
+static const renf_class* get_cone_renf_renf(PyObject* cone)
 {
     NumberFieldCone* cone_ptr = reinterpret_cast< NumberFieldCone* >(
         PyCapsule_GetPointer(cone, cone_name_renf));
@@ -799,23 +837,23 @@ static renf_class* get_cone_renf_renf(PyObject* cone)
 }
 #endif
 
-static PyObject* pack_cone(Cone< mpz_class >* C, void* dummy = nullptr)
+static PyObject* pack_cone(Cone< mpz_class >* C, const void* dummy = nullptr)
 {
     return PyCapsule_New(reinterpret_cast< void* >(C), cone_name,
                          &delete_cone_mpz);
 }
 
-static PyObject* pack_cone(Cone< long long >* C, void* dummy = nullptr)
+static PyObject* pack_cone(Cone< long long >* C, const void* dummy = nullptr)
 {
     return PyCapsule_New(reinterpret_cast< void* >(C), cone_name_long,
                          &delete_cone_long);
 }
 
 #ifdef ENFNORMALIZ
-static PyObject* pack_cone(Cone< renf_elem_class >* C, void* nf)
+static PyObject* pack_cone(Cone< renf_elem_class >* C, const void* nf)
 {
     NumberFieldCone* cone_ptr = new NumberFieldCone();
-    cone_ptr->nf = reinterpret_cast< renf_class* >(nf);
+    cone_ptr->nf = reinterpret_cast< const renf_class* >(nf);
     cone_ptr->cone = C;
     return PyCapsule_New(reinterpret_cast< void* >(cone_ptr), cone_name_renf,
                          &delete_cone_renf);
@@ -1007,14 +1045,14 @@ static PyObject* _NmzConeIntern_renf(PyObject* kwargs)
         return NULL;
     }
 
-
-    renf_class* renf;
     // number_field_data contains 3 entries: poly, var, emb
     // All are strings
     string poly = PyUnicodeToString(PySequence_GetItem(number_field_data, 0));
     string var = PyUnicodeToString(PySequence_GetItem(number_field_data, 1));
     string emb = PyUnicodeToString(PySequence_GetItem(number_field_data, 2));
-    renf = new renf_class(poly.c_str(), var.c_str(), emb.c_str());
+    // boost::intrusive_ptr<const renf_class>* renf = new boost::intrusive_ptr<const renf_class>; 
+    boost::intrusive_ptr<const renf_class> renf = renf_class::make(poly, var, emb);
+    const renf_class* my_renf = renf.get();
 
     map< InputType, vector< vector< renf_elem_class > > > input;
 
@@ -1036,7 +1074,7 @@ static PyObject* _NmzConeIntern_renf(PyObject* kwargs)
                 continue;
             vector< vector< renf_elem_class > > Mat;
             try {
-                prepare_nf_input(Mat, current_value, renf);
+                prepare_nf_input(Mat, current_value, my_renf);
             }
             catch (PyNormalizInputException& e) {
                 PyErr_SetString(PyNormaliz_cppError,
@@ -1050,9 +1088,9 @@ static PyObject* _NmzConeIntern_renf(PyObject* kwargs)
     }
 
     Cone< renf_elem_class >* C = new Cone< renf_elem_class >(input);
-    C->setRenf(renf);
-
-    PyObject* return_container = pack_cone(C, renf);
+    C->setRenf(my_renf);
+    
+    PyObject* return_container = pack_cone(C, my_renf);
 
     return return_container;
     FUNC_END
@@ -1320,7 +1358,7 @@ PyObject* _NmzModify(Cone<Integer>* cone, PyObject* args)
 }
 
 #ifdef ENFNORMALIZ
-PyObject* _NmzModify_Renf(Cone<renf_elem_class>* cone, renf_class* nf, PyObject* args)
+PyObject* _NmzModify_Renf(Cone<renf_elem_class>* cone, const renf_class* nf, PyObject* args)
 {
     string property = PyUnicodeToString( PyTuple_GetItem(args, 1) );
     PyObject* matrix_py = PyTuple_GetItem(args,2);
@@ -1359,7 +1397,7 @@ PyObject* _NmzModify_Outer(PyObject* self, PyObject* args)
 #ifdef ENFNORMALIZ
     else if (is_cone_renf(cone)) {
         Cone< renf_elem_class >* cone_ptr = get_cone_renf(cone);
-        renf_class* nf = get_cone_renf_renf(cone);
+        const renf_class* nf = get_cone_renf_renf(cone);
         return _NmzModify_Renf(cone_ptr, nf, args);
     }
 #endif
@@ -1591,7 +1629,7 @@ second is the projection third is the annihilator.
 
 template < typename Integer >
 static PyObject*
-_NmzResultImpl(Cone< Integer >* C, PyObject* prop_obj, void* nf = nullptr)
+_NmzResultImpl(Cone< Integer >* C, PyObject* prop_obj, const void* nf = nullptr)
 {
 
     string prop = PyUnicodeToString(prop_obj);
@@ -1725,6 +1763,10 @@ _NmzResultImpl(Cone< Integer >* C, PyObject* prop_obj, void* nf = nullptr)
         case libnormaliz::ConeProperty::AmbientAutomorphisms:
             return NmzAutomorphismsToPython(C->getAutomorphismGroup(
                 libnormaliz::ConeProperty::AmbientAutomorphisms));
+            
+        case libnormaliz::ConeProperty::InputAutomorphisms:
+            return NmzAutomorphismsToPython(C->getAutomorphismGroup(
+                libnormaliz::ConeProperty::InputAutomorphisms));
 
         case libnormaliz::ConeProperty::CombinatorialAutomorphisms:
             return NmzAutomorphismsToPython(C->getAutomorphismGroup(
@@ -1840,7 +1882,7 @@ MatrixHandler = NULL;
         Cone< renf_elem_class >* cone_ptr = get_cone_renf(cone);
         result = _NmzResultImpl(
             cone_ptr, prop,
-            reinterpret_cast< void* >(get_cone_renf_renf(cone)));
+            reinterpret_cast< const void* >(get_cone_renf_renf(cone)));
     }
 #endif
 
@@ -2094,6 +2136,45 @@ static PyObject* NmzSetFaceCodimBound(PyObject* self, PyObject* args)
     FUNC_END
 }
 
+static PyObject* NmzSetDecimalDigits(PyObject* self, PyObject* args)
+{
+
+    FUNC_BEGIN
+
+    PyObject* cone = PyTuple_GetItem(args, 0);
+
+    if (!is_cone(cone)) {
+        PyErr_SetString(PyNormaliz_cppError, "First argument must be a cone");
+        return NULL;
+    }
+
+    PyObject* digits_py = PyTuple_GetItem(args, 1);
+
+    TempSignalHandler tmpHandler; // use custom signal handler
+
+    int  overflow;
+    long digits = PyLong_AsLongLongAndOverflow(digits_py, &overflow);
+    if (is_cone_mpz(cone)) {
+        Cone< mpz_class >* cone_ptr = get_cone_mpz(cone);
+        cone_ptr->setDecimalDigits(digits);
+        Py_RETURN_TRUE;
+    }
+    else if (is_cone_long(cone)) {
+        Cone< long long >* cone_ptr = get_cone_long(cone);
+        cone_ptr->setDecimalDigits(digits);
+        Py_RETURN_TRUE;
+    }
+#ifdef ENFNORMALIZ
+    else {
+         Cone<renf_elem_class>* cone_ptr = get_cone_renf(cone);
+         cone_ptr->setDecimalDigits(digits);
+         Py_RETURN_TRUE;
+    }
+#endif
+
+    FUNC_END
+}
+
 /***************************************************************************
  *
  * Get Symmetrized cone
@@ -2420,9 +2501,9 @@ static PyObject* NmzGetRenfInfo(PyObject* self, PyObject* args)
         );
         return NULL;
     }
-    renf_class* renf = get_cone_renf_renf(cone_py);
+    const renf_class* renf = get_cone_renf_renf(cone_py);
     std::string minpoly_str;
-    minpoly_str = fmpq_poly_get_str_pretty(renf->get_renf()->nf->pol, renf->gen_name.c_str());
+    minpoly_str = fmpq_poly_get_str_pretty(renf->get_renf()->nf->pol, renf->gen_name().c_str());
     std::string res1 = arb_get_str(renf->get_renf()->emb, 64, 0);
     // long prec = renf->get_renf()->prec;
     return PyTuple_Pack(2, StringToPyUnicode(minpoly_str), StringToPyUnicode(res1));
@@ -2489,7 +2570,9 @@ static PyMethodDef PyNormaliz_cppMethods[] = {
      (PyCFunction)NmzSetNumberOfNormalizThreads, METH_VARARGS,
      "Sets the Normaliz thread limit"},
     {"NmzSetNrCoeffQuasiPol", (PyCFunction)NmzSetNrCoeffQuasiPol,
-     METH_VARARGS, "Sets the number of computed coefficients for the quasi-polynomial"},
+     METH_VARARGS, "Sets the number of computed coefficients for the quasi-polynomial"},     
+    {"NmzSetDecimalDigits", (PyCFunction)NmzSetDecimalDigits,
+     METH_VARARGS, "Sets the number of decimal digits for fixed precision"},
     {"NmzSetPolynomial", (PyCFunction)NmzSetPolynomial,
      METH_VARARGS, "Sets the polynomial for integration and weighted series"},
     {"NmzSetFaceCodimBound", (PyCFunction)NmzSetFaceCodimBound,
